/**
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *  GLite Data Transfer Agents: CancelTransfers Test Case
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: CancelPendingTestCase.cpp,v 1.8 2008/04/11 11:02:56 ptedesco Exp $
 *  Release: $Name:  $
 *
 */
#include "actions/vo/Cancel.h"
#include "DaoHelper.h"
#include "glite/data/transfer/agent/action/ActionFactory.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/dao/vo/FileDAO.h"
#include "glite/data/transfer/agent/dao/vo/JobDAO.h"
#include "glite/data/transfer/agent/dao/vo/DAOFactory.h"

#include "dao_test_utils.h"

#include <memory>
#include <boost/scoped_ptr.hpp>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

using namespace glite::data::transfer::agent;
using namespace glite::data::transfer::agent::action;
using namespace glite::data::transfer::agent::action::vo;
using namespace glite::data::transfer::agent::model;

using boost::scoped_ptr;

/**
 * CPPUnit TestCase
 */
class CancelPendingTestCase : public CppUnit::TestCase{
public:
    /**
     * set-up test case
     */
    void setUp ();
private:
    // Declare CPPUnit Methods
    // -------------------------------------------------------------------------
    CPPUNIT_TEST_SUITE( CancelPendingTestCase);
    CPPUNIT_TEST( testExecute );
    CPPUNIT_TEST_SUITE_END();

    // Test Methods
    // -------------------------------------------------------------------------

    /**
     * Test Execute Method
     */
    void testExecute();
    
private:
    /**
     * The Action Instance
     */
    scoped_ptr<Cancel> m_action;

};

/*
 * Register Suit
 */
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( CancelPendingTestCase, "CancelPendingTestCase" );

/*
 * Declare Constants. This unit test foresee at beginning
 * - 1 CANCELING Job with:                          => CANCELED
 *        job_id      = 4-...-01
 *        files       = 40000001 (PENDING)             CANCELED
 *                      40000002 (PENDING)             CANCELED
 *
 * - 1 CANCELING Job with:                          => CANCELED
 *        job_id      = 4-...-02
 *        files       = 40000003 (PENDING)             CANCELED
 *                      40000004 (WAITING)(req_4_1)    CANCELED
 *                      40000005 (HOLD)(req_4_1)       CANCELED
 *
 * - 1 CANCELING Job with:                          => CANCELING
 *        job_id      = 4-...-03
 *        files       = 40000006 (PENDING)             CANCELED
 *                      40000007 (ACTIVE) (req_4_2)    ACTIVE     
 *                      40000008 (WAITING)(req_4_2)    CANCELED
 * 
 * - 1 CANCELING Job with:                          => CANCELING
 *        job_id      = 4-...-04
 *        files       = 40000009 (DONE)    (req_4_3)    
 *                      40000010 (CANCELED)    
 *                      40000011 (FAILED)  (req_4_3)    
 */
namespace {
const char * const JOB1_ID   = "40000000-0000-0000-0000-000000000001";
const char * const JOB2_ID   = "40000000-0000-0000-0000-000000000002";
const char * const JOB3_ID   = "40000000-0000-0000-0000-000000000003";
const char * const JOB4_ID   = "40000000-0000-0000-0000-000000000004";
const char * const FILE1_ID  = "40000001";
const char * const FILE2_ID  = "40000002";
const char * const FILE3_ID  = "40000003";
const char * const FILE4_ID  = "40000004";
const char * const FILE5_ID  = "40000005";
const char * const FILE6_ID  = "40000006";
const char * const FILE7_ID  = "40000007";
const char * const FILE8_ID  = "40000008";
const char * const FILE9_ID  = "40000009";
const char * const FILE10_ID = "40000010";
const char * const FILE11_ID = "40000011";
const char * const VO1_NAME  = "CANCELPENDING_VO_1";
}

/**
 * setUp
 * set-up test case
 */
void CancelPendingTestCase::setUp(){
    
    // Create Action instance
    std::auto_ptr<Action> a(ActionFactory::instance()->create("glite:CancelPending"));
    CPPUNIT_ASSERT_MESSAGE("Null Action", a.get() != 0);
    CPPUNIT_ASSERT_MESSAGE("Invalid Action Class", dynamic_cast<Cancel*>(a.get()) != 0);

    // Store Action        
    m_action.reset(dynamic_cast<Cancel*>(a.release()));
    CPPUNIT_ASSERT_MESSAGE("Null Action", m_action.get() != 0);
        
}

/**
 * Test Execute Method
 */
void CancelPendingTestCase::testExecute(){
    
    // Set Context
    m_action->context(VO1_NAME,DaoHelper::voCtx());
    
    // Execute Actions
    m_action->execute();
    
    // Create FileDAO instance
    scoped_ptr<dao::vo::FileDAO> fileDAO(dao::vo::DAOFactory::getInstance()->createFileDAO(VO1_NAME, DaoHelper::voCtx()));
    CPPUNIT_ASSERT_MESSAGE("Null FileDAO", fileDAO.get() != 0);
    
    // Create JobDAO instance
    scoped_ptr<dao::vo::JobDAO> jobDAO(dao::vo::DAOFactory::getInstance()->createJobDAO(VO1_NAME, DaoHelper::voCtx()));
    CPPUNIT_ASSERT_MESSAGE("Null JobDAO", jobDAO.get() != 0);
    
    // Check The Transfers State using FileDAO
    // Get File instance (1) and Check it
    scoped_ptr<File> file(fileDAO->get(FILE1_ID));
    check_file(fileDAO,FILE1_ID,"FILE1",File::S_CANCELED);
    // Get File instance (2) and Check it
    check_file(fileDAO,FILE2_ID,"FILE2",File::S_CANCELED);
    // Get File instance (3) and Check it
    check_file(fileDAO,FILE3_ID,"FILE3",File::S_CANCELED);
    // Get File instance (4) and Check it
    check_file(fileDAO,FILE4_ID,"FILE4",File::S_CANCELED);
    // Get File instance (5) and Check it
    check_file(fileDAO,FILE5_ID,"FILE5",File::S_CANCELED);
    // Get File instance (6) and Check it
    check_file(fileDAO,FILE6_ID,"FILE6",File::S_CANCELED);
    // Get File instance (7) and Check it
    check_file(fileDAO,FILE7_ID,"FILE7",File::S_ACTIVE);
    // Get File instance (8) and Check it
    check_file(fileDAO,FILE8_ID,"FILE8",File::S_CANCELED);
    // Get File instance (9) and Check it
    check_file(fileDAO,FILE9_ID,"FILE9",File::S_DONE);
    // Get File instance (10) and Check it
    check_file(fileDAO,FILE10_ID,"FILE10",File::S_CANCELED);
    // Get File instance (11) and Check it
    check_file(fileDAO,FILE11_ID,"FILE11",File::S_FAILED);

    // Check The Jobs using JobDAO
    // Get Job instance (1) and Check it
    scoped_ptr<Job> job(jobDAO->get(JOB1_ID));
    check_job(jobDAO,JOB1_ID,"JOB1",Job::S_CANCELED);
    // Get Job instance (2) and Check it
    check_job(jobDAO,JOB2_ID,"JOB2",Job::S_CANCELED);
    // Get Job instance (3) and Check it
    check_job(jobDAO,JOB3_ID,"JOB3",Job::S_CANCELING);
    // Get Job instance (4) and Check it
    check_job(jobDAO,JOB4_ID,"JOB4",Job::S_CANCELING);
}
