#! /bin/bash
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://www.eu-egee.org/partners/ for details on the copyright holders.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#-------------------------------------------------------------------------------
# Unit tests for org.glite.data.transfer-agents
# Authors: Paolo Tedesco <paolo.tedesco@cern.ch>
# Version info: $Id: unit-test.sh,v 1.5 2009/07/02 15:21:17 molnarzs Exp $
# Release: $Name:  $
#-------------------------------------------------------------------------------

# check if the WORKSPACE environment variable is defined
if [ -z "${WORKSPACE}" ]; then
    echo "The WORKSPACE environment variable is not set."
    exit 1
fi

test_utils_module="org.glite.data.test-utils"
test_utils_dir="${WORKSPACE}/${test_utils_module}"

# source the test-common script
test_common_script="${test_utils_dir}/test-common.sh"
if [ ! -e ${test_common_script} ]; then
    echo "Checkout or update the ${test_utils_module} module."
    exit 1
fi
. ${test_common_script}

# source the test-proxy-utils script
test_proxy_utils_script="${test_utils_dir}/test-proxy-utils.sh"
if [ ! -e ${test_proxy_utils_script} ]; then
    echo "Checkout or update the ${test_utils_module} module."
    exit 1
fi
. ${test_proxy_utils_script}

# add org.gridsite.core paths
function add_gridsite_core_paths(){
    debug "adding gridsite.core paths"
    export GRIDSITE_CORE_LOCATION=$(get_glite_path "org.gridsite.core")
    addpath LD_LIBRARY_PATH ${GRIDSITE_CORE_LOCATION}/lib
}
add_gridsite_core_paths

#-------------------------------------------------------------------------------

MODULE_NAME="glite-data-transfer-agents"
MODULE_DIR="${WORKSPACE}/org.glite.data.transfer-agents"
MODULE_TEST_DIR="${GLITE_LOCATION}/share/test/${MODULE_NAME}"
MODULE_CONFIG_DIR="${GLITE_LOCATION}/share/config/${MODULE_NAME}"
MODULE_CONFIG_TEMPLATES="${MODULE_CONFIG_DIR}/config-templates"

AGENTS_COMMON_NAME="glite-data-agents-common"
AGENTS_COMMON_DIR="${WORKSPACE}/org.glite.data.agents-common"
AGENTS_COMMON_TEST_DIR="${GLITE_LOCATION}/share/test/${AGENTS_COMMON_NAME}"
AGENTS_COMMON_CONFIG_DIR="${GLITE_LOCATION}/share/config/${AGENTS_COMMON_NAME}"
AGENTS_COMMON_CONFIG_TEMPLATES="${AGENTS_COMMON_CONFIG_DIR}/config-templates"

SRM_UTIL_CPP_NAME="glite-data-srm-util-cpp"
SRM_UTIL_CPP_DIR="${WORKSPACE}/org.glite.data.srm-util-cpp"
SRM_UTIL_CPP_TEST_DIR="${GLITE_LOCATION}/share/test/${SRM_UTIL_CPP_NAME}"
SRM_UTIL_CPP_CONFIG_DIR="${GLITE_LOCATION}/share/config/${SRM_UTIL_CPP_NAME}"
SRM_UTIL_CPP_CONFIG_TEMPLATES="${SRM_UTIL_CPP_CONFIG_DIR}/config-templates"

# transfer-agents test scripts
. ${script_dir}/actions/unit-test.sh
. ${script_dir}/actions/srm_gridftp_split/unit-test.sh
. ${script_dir}/agents/unit-test.sh
. ${script_dir}/catalog/unit-test.sh
. ${script_dir}/config/unit-test.sh
. ${script_dir}/cred/unit-test.sh
. ${script_dir}/dao/unit-test.sh
. ${script_dir}/fsm/unit-test.sh
. ${script_dir}/namegen/unit-test.sh
. ${script_dir}/python/unit-test.sh
. ${script_dir}/regressions/13301/unit-test.sh
. ${script_dir}/regressions/13724/unit-test.sh
. ${script_dir}/regressions/30271/unit-test.sh
. ${script_dir}/regressions/31161/unit-test.sh
. ${script_dir}/regressions/unit-test.sh
. ${script_dir}/scheduler/unit-test.sh
. ${script_dir}/ts/unit-test.sh

# transfer-agents test specific variable
SCHEMA_DIR="${WORKSPACE}/org.glite.data.transfer-fts/config/schema/oracle"
DELEG_SCHEMA_DIR="${WORKSPACE}/org.glite.security.delegation-interface/interface/schema/oracle/"

# refactored out of "init_unit_test" to avoid creating certificates for tests 
# that don't need them
function init_libs_and_paths(){
    # add srm util cpp library paths for stub servers
    addpath LD_LIBRARY_PATH ${SRM_UTIL_CPP_TEST_DIR}/lib
    addpath LD_LIBRARY_PATH ${MODULE_TEST_DIR}/lib
    # add libraries paths
    add_gsoap_paths
    add_globus_paths
    add_log4cpp_paths
    add_glite_paths
    add_python_paths
    #add_oracle_paths
    # create test dirs
    mkdir -p ${MODULE_TEST_DIR}/{etc,bin,var/log}
}

# initialize tests
init_unit_test__DEPENDS="init_libs_and_paths"
function init_unit_test(){

    # create certificates
    create_fake_certificates || return 1
    # start local myproxy-server
    start_myproxy_server || return 1
    # call myproxy-init
    myproxy_init || return 1
    # initialize proxy
    grid_proxy_init || return 1
}

# copy schema to test location
function copy_schema_scripts(){
    info "Copying database schema sql scripts"
    local DEST_DIR="${MODULE_TEST_DIR}/bin"
    cp -v ${SCHEMA_DIR}/*.sql ${DEST_DIR}
    cp -v ${DELEG_SCHEMA_DIR}/oracle-schema.sql ${DEST_DIR}/oracle-deleg-schema.sql
    cp -v ${DELEG_SCHEMA_DIR}/oracle-drop.sql ${DEST_DIR}/oracle-deleg-drop.sql
    cp -v ${MODULE_DIR}/test/scripts/test-clear-database.sql ${DEST_DIR}
}

# the sqlplus command, to use directly when it's necessary to pass additional parameters
# to the script
export SQLPLUS_CMD="sqlplus -S ${ORA_USER}/${ORA_PWD}@${ORA_CONNECTSTRING}"

#
# RunSqlCmd
# ${1..n} = sql files
#
function RunSqlCmd(){
    debug "RunSqlCmd : ${#} scripts to execute"
    for ((i=1;i<=${#};i++)); do
        local sqlfile=${!i}
        debug "${SQLPLUS_CMD} < ${sqlfile}"
        if [ -f "${sqlfile}" ]; then
            ${SQLPLUS_CMD} < ${sqlfile}
        else
            error "File ${sqlfile} not found!"
            return 1
        fi
    done 
    return 0 
}

# clear the database
function clear_db(){
    # copy scripts just once
    run_test_function copy_schema_scripts || return ${?}
    # clear tables
    RunSqlCmd "${MODULE_TEST_DIR}/bin/test-clear-database.sql" || return ${?}
    if [ "0" == "${ORA_RECREATEDB}" ]; then
        info "Skipping drop and create tables statements."
    else
        # drop and recreate tables
        RunSqlCmd "${MODULE_TEST_DIR}/bin/oracle-drop.sql" \
            "${MODULE_TEST_DIR}/bin/oracle-deleg-drop.sql" \
            "${MODULE_TEST_DIR}/bin/oracle-schema.sql" \
            "${MODULE_TEST_DIR}/bin/oracle-deleg-schema.sql" || return ${?}
    fi 
    return 0 
}

# run all tests
test_all__DEPENDS="test_actions test_actions_split test_agents test_catalog test_config test_cred test_dao test_fsm test_namegen test_python test_regressions test_scheduler test_ts"
function test_all(){
    true
}

process_command_line ${*}

